<?php

namespace FiloBlu\Core\Plugin\Magento\Webapi\Model\Soap;

use Magento\Store\Model\Store;

/*
 * Plugin needed to fix SOAP WSDL urls
 * https://your.magento.host/soap/all?wsdl_list=1
 */
class Server
{

    /**
     * @var \Magento\Framework\App\AreaList
     */
    protected $areaList;
    /**
     * @var \Magento\Framework\Config\ScopeInterface
     */
    protected $scopeInterface;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $storeManagerInterface;
    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    protected $requestInterface;

    public function __construct(
        \Magento\Framework\App\AreaList $areaList,
        \Magento\Framework\Config\ScopeInterface $scopeInterface,
        \Magento\Store\Model\StoreManagerInterface $storeManagerInterface,
        \Magento\Framework\App\RequestInterface $requestInterface
    ){
        $this->areaList = $areaList;
        $this->scopeInterface = $scopeInterface;
        $this->storeManagerInterface = $storeManagerInterface;
        $this->requestInterface = $requestInterface;
    }

    public function afterGetEndpointUri(
        \Magento\Webapi\Model\Soap\Server $subject,
        $result
    ) {

        $front_name = $this->areaList->getFrontName($this->scopeInterface->getCurrentScope());
        // Not in SOAP scope? Do nothing!
        if ($front_name !== 'soap'){
            return $result;
        }

        // Stripping store code from base url
        $current_store_code = $this->storeManagerInterface->getStore()->getCode();
        $base_url = $this->storeManagerInterface->getStore()->getBaseUrl();
        $stripped_store_code_base_url = str_replace('/'.$current_store_code,'',$base_url);

        // StoreCode of original getEndpointUri function
        $storeCode = $this->storeManagerInterface->getStore()->getCode() === \Magento\Store\Model\Store::ADMIN_CODE
            ? \Magento\Webapi\Controller\PathProcessor::ALL_STORE_CODE
            : $this->storeManagerInterface->getStore()->getCode();

        // Rebuild
        $new_url = $stripped_store_code_base_url . $front_name . '/' . $storeCode;

        return $new_url;
    }
}
